#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import wx

class ListControler:
    """ Kísérlet egy minimális grafikus interface-nek a adatoktól és a rendszerlogikától való
        elválasztására az MVC (Model-View-Controller) pattern szellemében
        Az MVC (Model-View-Controller) pattern-ről lásd: http://ootips.org/mvc-pattern.html
        Megjegyzés: Az elnevezési konvenciók ebben a class-ban a PEP 8 elnevezési konvencióit követik,
        mivel ez a class többségében kívül esik a wxPython kontextusán
    """
    def __init__(self):
        self.list = []

    def get_list(self):
        """ A belső listára egy hivatkozást tesz lehetővé.
        """
        return self.list

    def add_element(self, position=0, text=""):
        """ Hozzáad egy elemet a listához, ha lehetséges
        """
        if len(self.list) >= position and position >= 0:
            self.list.insert(position, text)
        else:
            self.list.append(text)

    def del_element(self, position):
        """ Kivesz egy elemet a listából, ha lehetséges
        """
        if len(self.list) >= position and position >= 0:
            del self.list[position]

class MainWindow(wx.Frame):
    """ Az alkalmazás főablaka. Minden esetre csak egy lesz belőle.
    """
    def __init__(self, title):
        # A frame-nek 640*480-as kezdő mérete van és nem fog tudni e fölé menni
        wx.Frame.__init__(self, None, wx.ID_ANY, title, size=(640, 480))
        self.SetMinSize(self.GetSize())

        # A frame különböző widget-jeinek létrehozása
        self.panel = wx.Panel(self)
        self.lblEntry = wx.StaticText(self.panel, wx.ID_ANY, "Adatbevitel")

        # Az adatbevitel felügyelete természetesen a fókusszal kezdődik
        self.txtEntry = wx.TextCtrl(self.panel, wx.ID_ANY)
        self.txtEntry.SetFocus()

        # A lista inaktiválva van, mert kezdetben üres
        self.lstListe = wx.ListBox(self.panel, wx.ID_ANY)
        self.lstListe.Enable(False)

        # A Ment gomb a lista elé jön a tabulációval történő fókuszálás listájában
        # ráadásul a program főműveletét reprezentálja, amit az <Enter>
        # billentyű lenyomása aktivál
        self.cmdEnregistrer = wx.Button(self.panel, wx.ID_ANY, "Ment")
        self.cmdEnregistrer.MoveBeforeInTabOrder(self.lstListe)
        self.cmdEnregistrer.SetDefault()

        # A lista vezérlőgombjainak kezdetben inaktívaknak kell lenni
        self.cmdModifier = wx.Button(self.panel, wx.ID_ANY, "Módosít")
        self.cmdModifier.Enable(False)
        self.cmdSupprimer = wx.Button(self.panel, wx.ID_ANY, "Töröl")
        self.cmdSupprimer.Enable(False)
        self.cmdQuitter = wx.Button(self.panel, wx.ID_ANY, "Kilép")

        # A mnemonikus billentyűk asszociálása
        table = [(wx.ACCEL_CTRL, ord('E'), self.cmdEnregistrer.GetId()),
                (wx.ACCEL_CTRL, ord('M'), self.cmdModifier.GetId()),
                (wx.ACCEL_ALT, ord('S'), self.cmdSupprimer.GetId()),
                (wx.ACCEL_ALT, ord('Q'), self.cmdQuitter.GetId())]
        self.SetAcceleratorTable(wx.AcceleratorTable(table))

        # Egy karakterlánc, ami figyelmeztet a törlésre
        self.confirmSupprMess = "Valóban törölni akarja a " \
                            + " kiválasztott elemet ?"

        # Egy interface-től független lista létrehozása
        self.listObject = ListControler()

        # Az alapkiíratást és a szükséges esemény-metódus-widget összekapcsolást
        # lehetővé tevő két privát metódus
        self.__do_layout()
        self.__do_binding()
        self.modeModification = False

    def __do_layout(self):
        """ A widget-ek létrehozása és elrendezése az ablakban. Ez egy komplett 
            alkalmazása a sizer-ekre vonatkozó elméletnek.
        """
        # Ez az ablak két részre van felosztva. Az egyik rész az adatbevitel TextCtrl widget-je számára van
        # a másik rész a többi widget számára van
        mainSizer = wx.FlexGridSizer(rows=2)
        mainSizer.AddGrowableCol(0)
        mainSizer.AddGrowableRow(1)

        # Ez a sizer kezeli a felhasználó adatbeviteli widget-jeit
        # (txtEntry, cmdEnregistrer)
        entrySizer = wx.FlexGridSizer(cols=3, hgap=30)
        entrySizer.AddGrowableCol(1)
        entrySizer.Add(self.lblEntry, 0, wx.ALIGN_CENTER)
        entrySizer.Add(self.txtEntry, 0, wx.EXPAND)
        entrySizer.Add(self.cmdEnregistrer, 0, 0)

        # Ez a sizer kezeli azt, ami a listához kapcsolódik, 
        # valamint a Kilép gombot
        listSizer = wx.FlexGridSizer(cols=2, hgap=30)
        listSizer.AddGrowableCol(0)
        listSizer.AddGrowableRow(0)
        listSizer.Add(self.lstListe, 0, wx.EXPAND)

        # Ez a sizer teszi lehetővé, hogy megőrizzük a gombok oszlopokba rendezettségét
        listButtonsSizer = wx.FlexGridSizer(rows=3, vgap=10)
        listButtonsSizer.Add(self.cmdModifier, 0, 0)
        listButtonsSizer.Add(self.cmdSupprimer, 0, 0)
        listButtonsSizer.AddGrowableRow(2)
        listButtonsSizer.Add(self.cmdQuitter, 0, wx.ALIGN_BOTTOM)
        listSizer.Add(listButtonsSizer, 0, wx.EXPAND | wx.ALL, 0)
        mainSizer.Add(entrySizer, 0, wx.EXPAND | wx.ALL, 10)
        mainSizer.Add(listSizer, 0, wx.EXPAND | wx.ALL, 10)
        self.panel.SetAutoLayout(True)
        self.panel.SetSizer(mainSizer)
        self.Layout()

    def __do_binding(self):
        """ Itt hozzuk létre az erre az ablakra vonatkozó eseménykezelőket
        """
        self.Bind(wx.EVT_BUTTON, self.OnClickEnregistrer, self.cmdEnregistrer)
        self.Bind(wx.EVT_BUTTON, self.OnClickSupprimer, self.cmdSupprimer)
        self.Bind(wx.EVT_BUTTON, self.OnClickModifier, self.cmdModifier)
        self.Bind(wx.EVT_BUTTON, self.OnClickQuitter, self.cmdQuitter)
        self.Bind(wx.EVT_LISTBOX, self.OnListSelect, self.lstListe)

    def OnClickEnregistrer(self, event=None):
        """ Kattintás a "Ment" gombra
        """
        # Ha "Módosít" mód aktív, akkor módosítani kell 
        # a listából kiválasztott elemet,
        if self.modeModification:
           self.listObject.del_element(self.selPos)
           self.listObject.add_element(self.selPos,
                                    self.txtEntry.GetLineText(0))
           self.modeModification = False

        # ha nem, akkor a szöveget a lista végéhez kell fűzni
        else:
           self.listObject.add_element(text=self.txtEntry.GetLineText(0))

        # A lista szinkronizálása a kontroller listájával
        self.lstListe.Set(self.listObject.get_list())

        # Az adatbeviteli mező szövegének törlése, a fókusz visszakapása
        # és a lista aktívvá válik, hogy elemet válasszunk belőle
        self.txtEntry.SetValue("")
        self.txtEntry.SetFocus()
        self.lstListe.Enable(True)

    def OnClickModifier(self, event=None):
        """ Kattintás a "Modifier" gombra
        """
        # Aktiválja a Módosítás módot és kinyeri a kiválasztott elem indexét
        self.modeModification = True
        self.selPos = self.lstListe.GetSelection()

        # Az adatbeviteli mező felveszi a kiválasztott értéket és megkapja a fókuszt
        self.txtEntry.SetValue(self.lstListe.GetStringSelection())
        self.txtEntry.SetFocus()

        # Inaktiválja a listát és a listát vezérlő widget-eket,
        # hogy elkerüljük a felhasználó okozta hibákat
        self.lstListe.Enable(False)
        self.cmdModifier.Enable(False)
        self.cmdSupprimer.Enable(False)

    def OnClickSupprimer(self, event=None):
        """ Kattintás a "Töröl" gombra
        """
        # Egy jóváhagyást kérő üzenet kiíratása
        confirmSuppr = wx.MessageDialog(self, self.confirmSupprMess,
                                        style=wx.YES_NO | wx.NO_DEFAULT)

        # A felhasználó jóváhagyására törli az elemet a listából
        if confirmSuppr.ShowModal() == wx.ID_YES:
            self.listObject.del_element(self.lstListe.GetSelection())
            self.lstListe.Set(self.listObject.get_list())

    def OnClickQuitter(self, event=None):
        """ Kattintás a "Kilép" gombra
        """
        self.Close()

    def OnListSelect(self, event=None):
        """ Egy elem kiválasztása a listából
        """
        self.cmdModifier.Enable(True)
        self.cmdSupprimer.Enable(True)

class MainApp(wx.App):
    """ Az alkalmazás főablakát hozza létre
    """
    def OnInit(self):
        frame = MainWindow(title="Listaszerkesztő")
        self.SetTopWindow(frame)
        frame.Show()
        return 1

if __name__ == "__main__":
    app = MainApp()
    app.MainLoop()
